@@ -50,6 +50,10 @@ from opencl4py._cffi import (CL_DEVICE_TYPE_CPU,
                              CL_DEVICE_TYPE_CUSTOM,
                              CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE,
                              CL_QUEUE_PROFILING_ENABLE,
+                             CL_QUEUE_ON_DEVICE,
+                             CL_QUEUE_ON_DEVICE_DEFAULT,
+                             CL_QUEUE_PROPERTIES,
+                             CL_QUEUE_SIZE,
                              CL_MAP_READ,
                              CL_MAP_WRITE,
                              CL_MAP_WRITE_INVALIDATE_REGION,
@@ -59,6 +63,13 @@ from opencl4py._cffi import (CL_DEVICE_TYPE_CPU,
                              CL_MEM_USE_HOST_PTR,
                              CL_MEM_ALLOC_HOST_PTR,
                              CL_MEM_COPY_HOST_PTR,
+                             CL_MEM_HOST_NO_ACCESS,
+                             CL_MEM_SVM_FINE_GRAIN_BUFFER,
+                             CL_MEM_SVM_ATOMICS,
+                             CL_DEVICE_SVM_COARSE_GRAIN_BUFFER,
+                             CL_DEVICE_SVM_FINE_GRAIN_BUFFER,
+                             CL_DEVICE_SVM_FINE_GRAIN_SYSTEM,
+                             CL_DEVICE_SVM_ATOMICS,
                              CL_PROFILING_COMMAND_QUEUED,
                              CL_PROFILING_COMMAND_SUBMIT,
                              CL_PROFILING_COMMAND_START,
