@@ -126,8 +126,22 @@ CL_DEVICE_PREFERRED_INTEROP_USER_SYNC = 0x1048
 CL_DEVICE_PRINTF_BUFFER_SIZE = 0x1049
 CL_DEVICE_IMAGE_PITCH_ALIGNMENT = 0x104A
 CL_DEVICE_IMAGE_BASE_ADDRESS_ALIGNMENT = 0x104B
+CL_DEVICE_PIPE_MAX_ACTIVE_RESERVATIONS = 0x1056
+CL_DEVICE_PIPE_MAX_PACKET_SIZE = 0x1057
+CL_DEVICE_SVM_CAPABILITIES = 0x1053
+CL_DEVICE_SVM_COARSE_GRAIN_BUFFER = 1
+CL_DEVICE_SVM_FINE_GRAIN_BUFFER = 2
+CL_DEVICE_SVM_FINE_GRAIN_SYSTEM = 4
+CL_DEVICE_SVM_ATOMICS = 8
+CL_DEVICE_PREFERRED_PLATFORM_ATOMIC_ALIGNMENT = 0x1058
+CL_DEVICE_PREFERRED_GLOBAL_ATOMIC_ALIGNMENT = 0x1059
+CL_DEVICE_PREFERRED_LOCAL_ATOMIC_ALIGNMENT = 0x105A
 CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE = 1
 CL_QUEUE_PROFILING_ENABLE = 2
+CL_QUEUE_ON_DEVICE = 4
+CL_QUEUE_ON_DEVICE_DEFAULT = 8
+CL_QUEUE_PROPERTIES = 0x1093
+CL_QUEUE_SIZE = 0x1094
 CL_PROGRAM_BUILD_LOG = 0x1183
 CL_MAP_READ = 1
 CL_MAP_WRITE = 2
@@ -138,6 +152,9 @@ CL_MEM_READ_ONLY = 4
 CL_MEM_USE_HOST_PTR = 8
 CL_MEM_ALLOC_HOST_PTR = 16
 CL_MEM_COPY_HOST_PTR = 32
+CL_MEM_HOST_NO_ACCESS = 512
+CL_MEM_SVM_FINE_GRAIN_BUFFER = 1024
+CL_MEM_SVM_ATOMICS = 2048
 CL_PROFILING_COMMAND_QUEUED = 0x1280
 CL_PROFILING_COMMAND_SUBMIT = 0x1281
 CL_PROFILING_COMMAND_START = 0x1282
@@ -158,6 +175,12 @@ CL_KERNEL_CONTEXT = 0x1193
 CL_KERNEL_PROGRAM = 0x1194
 CL_KERNEL_ATTRIBUTES = 0x1195
 CL_BUFFER_CREATE_TYPE_REGION = 0x1220
+CL_KERNEL_GLOBAL_WORK_SIZE = 0x11B5
+CL_KERNEL_WORK_GROUP_SIZE = 0x11B0
+CL_KERNEL_COMPILE_WORK_GROUP_SIZE = 0x11B1
+CL_KERNEL_LOCAL_MEM_SIZE = 0x11B2
+CL_KERNEL_PREFERRED_WORK_GROUP_SIZE_MULTIPLE = 0x11B3
+CL_KERNEL_PRIVATE_MEM_SIZE = 0x11B4
 
 
 #: ffi parser
@@ -198,11 +221,13 @@ def initialize(backends=("libOpenCL.so", "OpenCL.dll")):
     typedef cl_uint  cl_kernel_info;
     typedef uint32_t cl_kernel_work_group_info;
     typedef uint64_t cl_command_queue_properties;
+    typedef uint64_t cl_queue_properties;
     typedef uint64_t cl_mem_flags;
     typedef uint32_t cl_bool;
     typedef uint64_t cl_map_flags;
     typedef uint32_t cl_profiling_info;
     typedef uint32_t cl_buffer_create_type;
+    typedef uint64_t cl_svm_mem_flags;
 
     typedef void* cl_platform_id;
     typedef void* cl_device_id;
@@ -214,6 +239,7 @@ def initialize(backends=("libOpenCL.so", "OpenCL.dll")):
     typedef void* cl_event;
 
     typedef intptr_t cl_context_properties;
+    typedef intptr_t cl_pipe_properties;
 
     cl_int clGetPlatformIDs(cl_uint num_entries,
                             cl_platform_id *platforms,
@@ -303,6 +329,11 @@ def initialize(backends=("libOpenCL.so", "OpenCL.dll")):
                                     cl_device_id device,
                                     cl_command_queue_properties properties,
                                     cl_int *errcode_ret);
+    cl_command_queue clCreateCommandQueueWithProperties(
+                                    cl_context context,
+                                    cl_device_id device,
+                                    const cl_queue_properties *properties,
+                                    cl_int *errcode_ret);
     cl_int clReleaseCommandQueue(cl_command_queue command_queue);
 
     cl_mem clCreateBuffer(cl_context context,
@@ -395,6 +426,36 @@ def initialize(backends=("libOpenCL.so", "OpenCL.dll")):
                                    size_t param_value_size,
                                    void *param_value,
                                    size_t *param_value_size_ret);
+
+    cl_mem clCreatePipe(cl_context context,
+                        cl_mem_flags flags,
+                        cl_uint pipe_packet_size,
+                        cl_uint pipe_max_packets,
+                        const cl_pipe_properties *properties,
+                        cl_int *errcode_ret);
+
+    void *clSVMAlloc(cl_context context,
+                     cl_svm_mem_flags flags,
+                     size_t size,
+                     unsigned int alignment);
+    void clSVMFree(cl_context context,
+                   void *svm_pointer);
+    cl_int clEnqueueSVMMap(cl_command_queue command_queue,
+                           cl_bool blocking_map,
+                           cl_map_flags map_flags,
+                           void *svm_ptr,
+                           size_t size,
+                           cl_uint num_events_in_wait_list,
+                           const cl_event *event_wait_list,
+                           cl_event *event);
+    cl_int clEnqueueSVMUnmap(cl_command_queue command_queue,
+                             void *svm_ptr,
+                             cl_uint  num_events_in_wait_list,
+                             const cl_event *event_wait_list,
+                             cl_event *event);
+    cl_int clSetKernelArgSVMPointer(cl_kernel kernel,
+                                    cl_uint arg_index,
+                                    const void *arg_value);
     """
 
     # Parse
