@@ -256,18 +256,40 @@ class Queue(CL):
         context: context associated with this queue.
         device: device associated with this queue.
     """
-    def __init__(self, context, device, flags):
+    def __init__(self, context, device, flags, properties=None):
+        """Creates the OpenCL command queue associated with the given device.
+
+        Parameters:
+            context: Context instance.
+            device: Device instance.
+            flags: flags for the command queue creation.
+            properties: dictionary of the OpenCL 2.0 queue properties.
+        """
         super(Queue, self).__init__()
         self._context = context
         self._device = device
         err = cl.ffi.new("cl_int *")
-        self._handle = self._lib.clCreateCommandQueue(
-            context.handle, device.handle, flags, err)
+        if properties is None or device.version < 2.0:
+            fnme = "clCreateCommandQueue"
+            self._handle = self._lib.clCreateCommandQueue(
+                context.handle, device.handle, flags, err)
+        else:
+            fnme = "clCreateCommandQueueWithProperties"
+            if properties is None and flags == 0:
+                props = cl.NULL
+            else:
+                if cl.CL_QUEUE_PROPERTIES not in properties and flags != 0:
+                    properties[cl.CL_QUEUE_PROPERTIES] = flags
+                props = cl.ffi.new("uint64_t[]", len(properties) * 2 + 1)
+                for i, kv in enumerate(sorted(properties.items())):
+                    props[i * 2] = kv[0]
+                    props[i * 2 + 1] = kv[1]
+            self._handle = self._lib.clCreateCommandQueueWithProperties(
+                context.handle, device.handle, props, err)
         if err[0]:
             self._handle = None
-            raise CLRuntimeError("clCreateCommandQueue() failed with "
-                                 "error %s" %
-                                 CL.get_error_description(err[0]),
+            raise CLRuntimeError("%s() failed with error %s" %
+                                 (fnme, CL.get_error_description(err[0])),
                                  err[0])
 
     @property
@@ -339,6 +361,7 @@ class Queue(CL):
             buf: Buffer object.
             flags: mapping flags.
             size: mapping size.
+            blocking: if the call would block until completion.
             offset: mapping offset.
             wait_for: list of the Event objects to wait.
             need_event: return Event object or not.
@@ -510,6 +533,61 @@ class Queue(CL):
                                  "error %s" % CL.get_error_description(n), n)
         return Event(event[0]) if event != cl.NULL else None
 
+    def svm_map(self, svm_ptr, flags, size, blocking=True,
+                wait_for=None, need_event=False):
+        """Enqueues a command that will allow the host to update a region
+        of a SVM buffer.
+
+        Parameters:
+            svm_ptr: SVM object or numpy array or direct cffi pointer.
+            flags: mapping flags.
+            size: mapping size (may be None if svm_ptr is a numpy array).
+            blocking: if the call would block until completion.
+            wait_for: list of the Event objects to wait.
+            need_event: return Event object or not.
+
+        Returns:
+            Event object or None if need_event == False.
+        """
+        event = cl.ffi.new("cl_event[]", 1) if need_event else cl.NULL
+        wait_list, n_events = CL.get_wait_list(wait_for)
+        if isinstance(svm_ptr, SVM):
+            ptr = svm_ptr.handle
+        else:
+            ptr, size = CL.extract_ptr_and_size(svm_ptr, size)
+        err = self._lib.clEnqueueSVMMap(
+            self.handle, blocking, flags, ptr, size,
+            n_events, wait_list, event)
+        if err:
+            raise CLRuntimeError("clEnqueueSVMMap() failed with error %s" %
+                                 CL.get_error_description(err), err)
+        return None if event == cl.NULL else Event(event[0])
+
+    def svm_unmap(self, svm_ptr, wait_for=None, need_event=True):
+        """Unmaps previously mapped SVM buffer.
+
+        Parameters:
+            svm_ptr: pointer that was specified in a previous call to svm_map.
+            wait_for: list of the Event objects to wait.
+            need_event: return Event object or not.
+
+        Returns:
+            Event object or None if need_event == False.
+        """
+        event = cl.ffi.new("cl_event[]", 1) if need_event else cl.NULL
+        wait_list, n_events = CL.get_wait_list(wait_for)
+        if isinstance(svm_ptr, SVM):
+            ptr = svm_ptr.handle
+        else:
+            ptr, _size = CL.extract_ptr_and_size(svm_ptr, 0)
+        err = self._lib.clEnqueueSVMUnmap(
+            self.handle, ptr, n_events, wait_list, event)
+        if err:
+            raise CLRuntimeError(
+                "clEnqueueSVMUnmap() failed with error %s" %
+                CL.get_error_description(err), err)
+        return Event(event[0]) if event != cl.NULL else None
+
     def flush(self):
         """Flushes the queue.
         """
@@ -651,6 +729,90 @@ class skip(object):
         self._number = value
 
 
+class WorkGroupInfo(CL):
+    """Some information about the kernel concerning the specified device.
+    """
+    def __init__(self, kernel, device):
+        super(WorkGroupInfo, self).__init__()
+        self._kernel = kernel
+        self._device = device
+
+    @property
+    def kernel(self):
+        return self._kernel
+
+    @property
+    def device(self):
+        return self._device
+
+    @property
+    def global_work_size(self):
+        """Returns the maximum global size that can be used to execute a kernel
+           on this device.
+
+        Raises:
+            CLRuntimeError: when device is not a custom device or
+                            kernel is not a built-in kernel.
+        """
+        buf = cl.ffi.new("size_t[]", 3)
+        self._get_info(cl.CL_KERNEL_GLOBAL_WORK_SIZE, buf)
+        return int(buf[0]), int(buf[1]), int(buf[2])
+
+    @property
+    def work_group_size(self):
+        """Returns the maximum global size that can be used to execute a kernel
+           on this device.
+        """
+        buf = cl.ffi.new("size_t *")
+        self._get_info(cl.CL_KERNEL_WORK_GROUP_SIZE, buf)
+        return int(buf[0])
+
+    @property
+    def compile_work_group_size(self):
+        """Returns the work-group size specified by the
+           __attribute__((reqd_work_group_size(X, Y, Z))) qualifier.
+        """
+        buf = cl.ffi.new("size_t[]", 3)
+        self._get_info(cl.CL_KERNEL_COMPILE_WORK_GROUP_SIZE, buf)
+        return int(buf[0]), int(buf[1]), int(buf[2])
+
+    @property
+    def local_mem_size(self):
+        """Returns the amount of local memory in bytes being used by a kernel.
+        """
+        buf = cl.ffi.new("uint64_t *")
+        self._get_info(cl.CL_KERNEL_LOCAL_MEM_SIZE, buf)
+        return int(buf[0])
+
+    @property
+    def preferred_work_group_size_multiple(self):
+        """Returns the preferred multiple of workgroup size for launch.
+        """
+        buf = cl.ffi.new("size_t *")
+        self._get_info(cl.CL_KERNEL_PREFERRED_WORK_GROUP_SIZE_MULTIPLE, buf)
+        return int(buf[0])
+
+    @property
+    def private_mem_size(self):
+        """Returns the minimum amount of private memory, in bytes,
+           used by each workitem in the kernel.
+        """
+        buf = cl.ffi.new("uint64_t *")
+        self._get_info(cl.CL_KERNEL_PRIVATE_MEM_SIZE, buf)
+        return int(buf[0])
+
+    def _get_info(self, code, buf):
+        sz = cl.ffi.new("size_t *")
+        err = self._lib.clGetKernelWorkGroupInfo(
+            self.kernel.handle, self.device.handle, code,
+            cl.ffi.sizeof(buf), buf, sz)
+        if err:
+            raise CLRuntimeError(
+                "clGetKernelWorkGroupInfo() failed with error %s" %
+                CL.get_error_description(err), err)
+        return sz[0]
+
+
 class Kernel(CL):
     """Holds OpenCL kernel.
 
@@ -704,6 +866,9 @@ class Kernel(CL):
         self._get_kernel_info(cl.CL_KERNEL_ATTRIBUTES, buf)
         return cl.ffi.string(buf).decode("utf-8", "replace").strip()
 
+    def get_work_group_info(self, device):
+        return WorkGroupInfo(self, device)
+
     def set_arg(self, idx, vle, size=None):
         """Sets kernel argument.
 
@@ -717,7 +882,7 @@ class Kernel(CL):
                  - may be cffi pointer also, in such case size should be set.
             size: size of the vle (may be None for buffers and scalars).
         """
-        if isinstance(vle, Buffer):
+        if isinstance(vle, Buffer) or isinstance(vle, Pipe):
             arg_value = cl.ffi.new("cl_mem[]", 1)
             arg_value[0] = vle.handle
             arg_size = cl.ffi.sizeof("cl_mem")
@@ -733,8 +898,10 @@ class Kernel(CL):
             if size is None:
                 raise ValueError("size should be set in case of cffi pointer")
             arg_size = size
+        elif isinstance(vle, SVM):
+            return self.set_arg_svm(idx, vle)
         else:
-            raise ValueError("vle should be of type Buffer, "
+            raise ValueError("vle should be of type Buffer, Pipe, SVM, "
                              "numpy array, cffi pointer or None "
                              "in Kernel::set_arg()")
         n = self._lib.clSetKernelArg(self.handle, idx, arg_size, arg_value)
@@ -744,6 +911,23 @@ class Kernel(CL):
                                          CL.get_error_description(n)),
                                  n)
 
+    def set_arg_svm(self, idx, svm_ptr):
+        """Sets SVM pointer as the kernel argument.
+
+        Parameters:
+            idx: index of the kernel argument (zero-based).
+            svm_ptr: SVM object or numpy array or direct cffi pointer.
+        """
+        if isinstance(svm_ptr, SVM):
+            ptr = svm_ptr.handle
+        else:
+            ptr, _size = CL.extract_ptr_and_size(svm_ptr, 0)
+        err = self._lib.clSetKernelArgSVMPointer(self.handle, idx, ptr)
+        if err:
+            raise CLRuntimeError(
+                "clSetKernelArgSVMPointer(%d, %s) failed with error %s" %
+                (idx, repr(svm_ptr), CL.get_error_description(err)), err)
+
     def set_args(self, *args):
         i = 0
         for arg in args:
@@ -766,8 +950,8 @@ class Kernel(CL):
 
     def _get_kernel_info(self, code, buf):
         sz = cl.ffi.new("size_t *")
-        err = self._lib.clGetKernelInfo(self.handle, code,
-                                        cl.ffi.sizeof(buf), buf, sz)
+        err = self._lib.clGetKernelInfo(
+            self.handle, code, cl.ffi.sizeof(buf), buf, sz)
         if err:
             raise CLRuntimeError("clGetKernelInfo() failed with error %s" %
                                  CL.get_error_description(err), err)
@@ -996,6 +1180,115 @@ class Program(CL):
         self.release()
 
 
+class Pipe(CL):
+    """Holds OpenCL pipe.
+
+    Attributes:
+        context: Context object associated with this pipe.
+        flags: flags for a pipe;
+               as of OpenCL 2.0 only CL_MEM_READ_ONLY, CL_MEM_WRITE_ONLY,
+               CL_MEM_READ_WRITE, and CL_MEM_HOST_NO_ACCESS can be specified
+               when creating a pipe object (0 defaults to CL_MEM_READ_WRITE).
+        packet_size: size in bytes of a pipe packet (must be greater than 0).
+        max_packets: maximum number of packets the pipe can hold
+                     (must be greater than 0).
+    """
+    def __init__(self, context, flags, packet_size, max_packets):
+        super(Pipe, self).__init__()
+        self._context = context
+        self._flags = flags
+        self._packet_size = packet_size
+        self._max_packets = max_packets
+        err = cl.ffi.new("cl_int *")
+        self._handle = self._lib.clCreatePipe(
+            context.handle, flags, packet_size, max_packets, cl.NULL, err)
+        if err[0]:
+            self._handle = None
+            raise CLRuntimeError("clCreatePipe() failed with error %s" %
+                                 CL.get_error_description(err[0]), err[0])
+
+    @property
+    def context(self):
+        return self._context
+
+    @property
+    def flags(self):
+        return self._flags
+
+    @property
+    def packet_size(self):
+        return self._packet_size
+
+    @property
+    def max_packets(self):
+        return self._max_packets
+
+    def release(self):
+        if self.handle is not None:
+            self._lib.clReleaseMemObject(self.handle)
+            self._handle = None
+
+    def __del__(self):
+        self.release()
+
+
+class SVM(CL):
+    """Holds shared virtual memory (SVM) buffer.
+
+    Attributes:
+        handle: pointer to the created buffer.
+        context: Context object associated with this buffer.
+        flags: flags for a buffer.
+        size: size in bytes of the SVM buffer to be allocated.
+        alignment: the minimum alignment in bytes (can be 0).
+    """
+    def __init__(self, context, flags, size, alignment=0):
+        super(SVM, self).__init__()
+        self._context = context
+        self._flags = flags
+        self._size = size
+        self._alignment = alignment
+        self._handle = self._lib.clSVMAlloc(
+            context.handle, flags, size, alignment)
+        if self._handle == cl.NULL:
+            self._handle = None
+            raise CLRuntimeError("clSVMAlloc() failed", -30)
+
+    @property
+    def context(self):
+        return self._context
+
+    @property
+    def flags(self):
+        return self._flags
+
+    @property
+    def size(self):
+        return self._size
+
+    @property
+    def alignment(self):
+        return self._alignment
+
+    @property
+    def buffer(self):
+        """Returns buffer object from this SVM pointer.
+
+        You can supply it to numpy.frombuffer() for example,
+        but be sure that destructor of an SVM object is called
+        after the last access to that numpy array.
+        """
+        return cl.ffi.buffer(self.handle, self.size)
+
+    def release(self):
+        if self.handle is not None and self.context.handle is not None:
+            self._lib.clSVMFree(self.context.handle, self.handle)
+            self._handle = None
+
+    def __del__(self):
+        self.release()
+
+
 class Context(CL):
     """Holds OpenCL context.
 
@@ -1038,18 +1331,19 @@ class Context(CL):
         """
         return self._devices
 
-    def create_queue(self, device, flags=0):
+    def create_queue(self, device, flags=0, properties=None):
         """Creates Queue object for the supplied device.
 
         Parameters:
             device: Device object.
             flags: queue flags (for example
                                 CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE).
+            properties: dictionary of OpenCL 2.0 queue properties.
 
         Returns:
             Queue object.
         """
-        return Queue(self, device, flags)
+        return Queue(self, device, flags, properties)
 
     def create_buffer(self, flags, host_array=None, size=None):
         """Creates Buffer object based on host_array.
@@ -1080,6 +1374,38 @@ class Context(CL):
         return Program(self, self.devices if devices is None else devices,
                        src, include_dirs, options, binary)
 
+    def create_pipe(self, flags, packet_size, max_packets):
+        """Creates OpenCL 2.0 pipe.
+
+        Parameters:
+            flags: flags for a pipe;
+                   as of OpenCL 2.0 only CL_MEM_READ_ONLY, CL_MEM_WRITE_ONLY,
+                   CL_MEM_READ_WRITE, and CL_MEM_HOST_NO_ACCESS
+                   can be specified when creating a pipe object
+                   (0 defaults to CL_MEM_READ_WRITE).
+            packet_size: size in bytes of a pipe packet
+                         (must be greater than 0).
+            max_packets: maximum number of packets the pipe can hold
+                         (must be greater than 0).
+        """
+        return Pipe(self, flags, packet_size, max_packets)
+
+    def svm_alloc(self, flags, size, alignment=0):
+        """Allocates shared virtual memory (SVM) buffer.
+
+        Parameters:
+            flags: flags for a buffer;
+                   (CL_MEM_READ_WRITE, CL_MEM_WRITE_ONLY,
+                    CL_MEM_READ_ONLY, CL_MEM_SVM_FINE_GRAIN_BUFFER,
+                    CL_MEM_SVM_ATOMICS).
+            size: size in bytes of the SVM buffer to be allocated.
+            alignment: the minimum alignment in bytes,
+                       it must be a power of two up to the largest
+                       data type supported by the OpenCL device,
+                       0 defaults to the largest supported alignment.
+        """
+        return SVM(self, flags, size, alignment)
+
     def release(self):
         if self.handle is not None:
             self._lib.clReleaseContext(self.handle)
@@ -1417,6 +1743,34 @@ class Device(CL):
             return None
         return list(value)
 
+    @property
+    def pipe_max_packet_size(self):
+        return self._get_device_info_int(cl.CL_DEVICE_PIPE_MAX_PACKET_SIZE)
+
+    @property
+    def pipe_max_active_reservations(self):
+        return self._get_device_info_int(
+            cl.CL_DEVICE_PIPE_MAX_ACTIVE_RESERVATIONS)
+
+    @property
+    def svm_capabilities(self):
+        return self._get_device_info_int(cl.CL_DEVICE_SVM_CAPABILITIES)
+
+    @property
+    def preferred_platform_atomic_alignment(self):
+        return self._get_device_info_int(
+            cl.CL_DEVICE_PREFERRED_PLATFORM_ATOMIC_ALIGNMENT)
+
+    @property
+    def preferred_global_atomic_alignment(self):
+        return self._get_device_info_int(
+            cl.CL_DEVICE_PREFERRED_GLOBAL_ATOMIC_ALIGNMENT)
+
+    @property
+    def preferred_local_atomic_alignment(self):
+        return self._get_device_info_int(
+            cl.CL_DEVICE_PREFERRED_LOCAL_ATOMIC_ALIGNMENT)
+
     def _get_device_info_bool(self, name):
         value = cl.ffi.new("cl_bool[]", 1)
         err = self._lib.clGetDeviceInfo(self._handle, name,
