@@ -67,6 +67,11 @@ class Test(unittest.TestCase):
         self.assertEqual(cl.CL_DEVICE_TYPE_ACCELERATOR, 8)
         self.assertEqual(cl.CL_DEVICE_TYPE_CUSTOM, 16)
         self.assertEqual(cl.CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE, 1)
+        self.assertEqual(cl.CL_QUEUE_PROFILING_ENABLE, 2)
+        self.assertEqual(cl.CL_QUEUE_ON_DEVICE, 4)
+        self.assertEqual(cl.CL_QUEUE_ON_DEVICE_DEFAULT, 8)
+        self.assertEqual(cl.CL_QUEUE_PROPERTIES, 0x1093)
+        self.assertEqual(cl.CL_QUEUE_SIZE, 0x1094)
         self.assertEqual(cl.CL_MAP_READ, 1)
         self.assertEqual(cl.CL_MAP_WRITE, 2)
         self.assertEqual(cl.CL_MAP_WRITE_INVALIDATE_REGION, 4)
@@ -76,6 +81,17 @@ class Test(unittest.TestCase):
         self.assertEqual(cl.CL_MEM_USE_HOST_PTR, 8)
         self.assertEqual(cl.CL_MEM_ALLOC_HOST_PTR, 16)
         self.assertEqual(cl.CL_MEM_COPY_HOST_PTR, 32)
+        self.assertEqual(cl.CL_MEM_HOST_NO_ACCESS, 512)
+        self.assertEqual(cl.CL_MEM_SVM_FINE_GRAIN_BUFFER, 1024)
+        self.assertEqual(cl.CL_MEM_SVM_ATOMICS, 2048)
+        self.assertEqual(cl.CL_DEVICE_SVM_COARSE_GRAIN_BUFFER, 1)
+        self.assertEqual(cl.CL_DEVICE_SVM_FINE_GRAIN_BUFFER, 2)
+        self.assertEqual(cl.CL_DEVICE_SVM_FINE_GRAIN_SYSTEM, 4)
+        self.assertEqual(cl.CL_DEVICE_SVM_ATOMICS, 8)
+        self.assertEqual(cl.CL_PROFILING_COMMAND_QUEUED, 0x1280)
+        self.assertEqual(cl.CL_PROFILING_COMMAND_SUBMIT, 0x1281)
+        self.assertEqual(cl.CL_PROFILING_COMMAND_START, 0x1282)
+        self.assertEqual(cl.CL_PROFILING_COMMAND_END, 0x1283)
 
     def test_dump_devices(self):
         platforms = cl.Platforms()
@@ -126,6 +142,18 @@ class Test(unittest.TestCase):
         self.assertGreater(dev.preferred_vector_width_int, 0)
         self.assertGreater(dev.max_work_group_size, 1)
         self.assertTrue(dev.available)
+        try:
+            self.assertTrue(type(dev.pipe_max_active_reservations) == int)
+            self.assertTrue(type(dev.pipe_max_packet_size) == int)
+            self.assertTrue(type(dev.svm_capabilities) == int)
+            self.assertTrue(
+                type(dev.preferred_platform_atomic_alignment) == int)
+            self.assertTrue(type(dev.preferred_global_atomic_alignment) == int)
+            self.assertTrue(type(dev.preferred_local_atomic_alignment) == int)
+        except cl.CLRuntimeError as e:
+            if dev.version >= 2.0:
+                raise
+            self.assertEqual(e.code, -30)
 
     def test_program_info(self):
         platforms = cl.Platforms()
@@ -166,38 +194,41 @@ class Test(unittest.TestCase):
         binary = prg.binaries[0]
         prg = ctx.create_program([binary], binary=True)
         krn = prg.get_kernel("test")
+        del krn
+
+    def test_set_kernel_args(self):
+        import numpy
 
-    def set_kernel_args(self):
         platforms = cl.Platforms()
         ctx = platforms.create_some_context()
         prg = ctx.create_program(self.src_test, self.include_dirs)
         krn = prg.get_kernel("test")
         queue = ctx.create_queue(ctx.devices[0])
-        global_size = [a.size]
+        global_size = [12345]
         local_size = None
 
         krn.set_args(cl.skip(3))
-        self.assertRaises(CLRuntimeError,
-                          queue.execute_kernel(krn, global_size, local_size))
-        krn.set_args(cl.skip. cl.skip, cl.skip)
-        self.assertRaises(CLRuntimeError,
-                          queue.execute_kernel(krn, global_size, local_size))
-        krn.set_args(cl.skip(1). cl.skip(1), cl.skip(1))
-        self.assertRaises(CLRuntimeError,
-                          queue.execute_kernel(krn, global_size, local_size))
+        self.assertRaises(cl.CLRuntimeError,
+                          queue.execute_kernel, krn, global_size, local_size)
+        krn.set_args(cl.skip, cl.skip, cl.skip)
+        self.assertRaises(cl.CLRuntimeError,
+                          queue.execute_kernel, krn, global_size, local_size)
+        krn.set_args(cl.skip(1), cl.skip(1), cl.skip(1))
+        self.assertRaises(cl.CLRuntimeError,
+                          queue.execute_kernel, krn, global_size, local_size)
         krn.set_args(cl.skip(1000))
-        self.assertRaises(CLRuntimeError,
-                          queue.execute_kernel(krn, global_size, local_size))
+        self.assertRaises(cl.CLRuntimeError,
+                          queue.execute_kernel, krn, global_size, local_size)
         self.assertRaises(ValueError, cl.skip, 0)
         self.assertRaises(ValueError, cl.skip, -1)
 
         c = numpy.array([1.2345], dtype=numpy.float32)
         krn.set_args(cl.skip(2), c)
-        self.assertRaises(CLRuntimeError,
-                          queue.execute_kernel(krn, global_size, local_size))
+        self.assertRaises(cl.CLRuntimeError,
+                          queue.execute_kernel, krn, global_size, local_size)
         krn.set_args(cl.skip, cl.skip, c)
-        self.assertRaises(CLRuntimeError,
-                          queue.execute_kernel(krn, global_size, local_size))
+        self.assertRaises(cl.CLRuntimeError,
+                          queue.execute_kernel, krn, global_size, local_size)
 
     def test_api_numpy(self):
         import numpy
@@ -603,6 +634,107 @@ class Test(unittest.TestCase):
         self.assertLess(numpy.fabs(aa - d).max(), 0.0001,
                         "Incorrect result after read_buffer")
 
+    def test_create_queue_with_properties(self):
+        ctx = cl.Platforms().create_some_context()
+        try:
+            queue = ctx.create_queue(
+                ctx.devices[0],
+                cl.CL_QUEUE_ON_DEVICE |
+                cl.CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE,
+                properties={cl.CL_QUEUE_SIZE: 64})
+            del queue
+        except cl.CLRuntimeError:
+            if ctx.devices[0].version >= 2.0:
+                raise
+            return
+        queue = ctx.create_queue(
+            ctx.devices[0],
+            properties={cl.CL_QUEUE_SIZE: 64,
+                        cl.CL_QUEUE_PROPERTIES:
+                        cl.CL_QUEUE_ON_DEVICE |
+                        cl.CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE})
+        del queue
+
+    def test_work_group_info(self):
+        ctx = cl.Platforms().create_some_context()
+        prg = ctx.create_program(self.src_test, self.include_dirs)
+        krn = prg.get_kernel("test")
+        info = krn.get_work_group_info(ctx.devices[0])
+
+        self.assertRaises(cl.CLRuntimeError, getattr, info, "global_work_size")
+
+        for vle in (info.compile_work_group_size,):
+            self.assertIsInstance(vle, tuple)
+            self.assertEqual(len(vle), 3)
+            for x in vle:
+                self.assertIsInstance(x, int)
+                self.assertGreaterEqual(x, 0)
+
+        for vle in (info.work_group_size, info.local_mem_size,
+                    info.preferred_work_group_size_multiple,
+                    info.private_mem_size):
+            self.assertIsInstance(vle, int)
+            self.assertGreaterEqual(vle, 0)
+
+    def test_create_pipe(self):
+        ctx = cl.Platforms().create_some_context()
+        if ctx.devices[0].version < 2.0:
+            return
+        pipe = ctx.create_pipe(0, 8, 16)
+        del pipe
+        pipe = ctx.create_pipe(cl.CL_MEM_READ_WRITE, 8, 16)
+        prg = ctx.create_program("""
+            __kernel void test(__write_only pipe int p) {
+                int x = 0;
+                write_pipe(p, &x);
+            }
+            """, options="-cl-std=CL2.0")
+        krn = prg.get_kernel("test")
+        krn.set_arg(0, pipe)
+        del krn
+        del prg
+        del pipe
+
+    def test_svm_alloc(self):
+        ctx = cl.Platforms().create_some_context()
+        if ctx.devices[0].version < 2.0:
+            return
+        svm = ctx.svm_alloc(cl.CL_MEM_READ_WRITE, 4096)
+        svm.release()
+        self.assertIsNone(svm.handle)
+        del svm
+        svm = ctx.svm_alloc(cl.CL_MEM_READ_WRITE, 4096)
+        prg = ctx.create_program("""
+            __kernel void test(__global void *p) {
+                __global int *ptr = (__global int*)p;
+                *ptr += 1;
+            }
+            """, options="-cl-std=CL2.0")
+        krn = prg.get_kernel("test")
+        krn.set_arg(0, svm)
+        krn.set_arg_svm(0, svm)
+        queue = ctx.create_queue(ctx.devices[0])
+        queue.svm_map(svm, cl.CL_MAP_WRITE_INVALIDATE_REGION, 4)
+        p = cl.ffi.cast("int*", svm.handle)
+        p[0] = 2
+        queue.svm_unmap(svm)
+        queue.execute_kernel(krn, [1], None)
+        queue.svm_map(svm, cl.CL_MAP_READ, 4)
+        self.assertEqual(p[0], 3)
+        # always ensure that the last unmap had completed before
+        # the svm destructor
+        queue.svm_unmap(svm).wait()
+        try:
+            import numpy
+            a = numpy.frombuffer(svm.buffer, dtype=numpy.int32)
+            queue.execute_kernel(krn, [1], None)
+            queue.svm_map(svm, cl.CL_MAP_READ, 4)
+            self.assertEqual(a[0], 4)
+            queue.svm_unmap(svm).wait()
+        except ImportError:
+            pass
+        del svm  # svm destructor here
+
 
 if __name__ == "__main__":
     unittest.main()
